#
# Copyright (c) Daniel Sheffield 2023
# All rights reserved
#
# THIS SOFTWARE IS PROVIDED AS IS WITHOUT WARRANTY
import os
from threading import Thread
from bottle import route, request, response, template, static_file
from psycopg import connect

from .route_decorators import normalize, poison, cursor
from .query_to_xml import get_categories, get_groups, get_products, get_tags
from .CachedLoadingPage import CachedLoadingPage
from .Cache import Cache
from . import trend as worker

host = f"host={os.getenv('HOST')}"
db = f"dbname={os.getenv('DB', 'grocery')}"
user = f"user={os.getenv('USER', 'das')}"
password = f"password={os.getenv('PASSWORD','')}"
if not password.split('=',1)[1]:
    password = ''
conn = connect(f"{host} {db} {user} {password}")

CACHE = Cache(10)

@route('/grocery/static/<filename:path>')
def send_static(filename):
    return static_file(filename, root='app/rest/static')


@route('/grocery/trend')
@poison(cache=CACHE)
@normalize
def trend():
    page = CACHE.get(request.query_string)
    _, _, path, *_ = request.urlparts
    return page if page else CACHE.add(request.query_string, CachedLoadingPage(
        template("loading", progress=[]),
        lambda queue: Thread(target=worker.trend, args=(
            queue, conn, path, request.query
        )).start()
    ))


@route('/grocery/groups')
@normalize
@cursor(connection=conn)
def groups(cur):
    response.content_type = 'application/xhtml+xml; charset=utf-8'
    return get_groups(cur, request.query)


@route('/grocery/categories')
@normalize
@cursor(connection=conn)
def categories(cur):
    response.content_type = 'application/xhtml+xml; charset=utf-8'
    return get_categories(cur, request.query)


@route('/grocery/products')
@normalize
@cursor(connection=conn)
def products(cur):
    response.content_type = 'application/xhtml+xml; charset=utf-8'
    return get_products(cur, request.query)


@route('/grocery/tags')
@normalize
@cursor(connection=conn)
def tags(cur):
    response.content_type = 'application/xhtml+xml; charset=utf-8'
    return get_tags(cur, request.query)
